import argparse
import wikipedia_tools.scraper.revision_extractor as re
import wikipedia_tools.scraper.category_extractor as ce
import json
from functools import reduce

REVISIONS_FILE_POSTFIX = '_revisions.jsonl'

def read_pages_file(pages_file):
    with open(pages_file) as f:
        lines = f.readlines()

    return lines

def main():
    parser = argparse.ArgumentParser(prog ='Wikipedia Parser',
                                     description ='Parser to extract Wikipedia discourse.')
  
    parser.add_argument('-c', '--categories', type = str, nargs ='*',
                        help ='Categories to parse.')

    parser.add_argument('-p', '--pages_file', type = str,
                        help ='File with a list of pages to parse. One page per line.')

    parser.add_argument('-o', '--outfile_prefix', type = str, required=True,
                        help ='Prefix for output files.')

    parser.add_argument('-g', action='store_true', 
                        help = 'Flag indicating if a revision and pagelink graph should be extracted')

    args = parser.parse_args()

    if args.pages_file:
        pages = read_pages_file(args.pages_file)
    else:
        pages = []

    if args.categories:
        pages = set(pages + reduce(list.__add__, [ce.get_category_pages(c) for c in args.categories]))

    if len(pages) == 0:
        raise Exception('No pages to parse. Please specify a proper category or a file with page names.')
    else:
        print('parsing ' + str(len(pages)) + '...')
        revisions_df = re.get_revisions_all_pages(pages, 'en')
            
        with open(args.outfile_prefix + REVISIONS_FILE_POSTFIX, 'w') as f:
            for r in revisions_df:
                f.write(json.dumps(r) + "\n")

if __name__ == '__main__':
    main()


import importlib
import pathlib
import  os
from IPy import IP
from datetime import datetime
def create_folder(folder_path):
    pathlib.Path(folder_path).mkdir(parents=True, exist_ok=True)

def file_exists(file_path):
    return os.path.isfile(file_path)

def get_alphanumeric(wiki_page_title):
    return ''.join(c for c in wiki_page_title if (c.isalnum() or c == ' ')).lower().replace(' ', '-')

def is_ip(s):
    valid = True
    try:
        IP(s)
    except:
        valid = False
    return valid


def get_last_month(decreased_month=0):
    now = datetime.now()
    last_month = now.replace(month=(now.month-decreased_month),
                      day=1, hour = 0, minute = 0, second=0, microsecond=0
                     )
    print(last_month.strftime("%Y-%m-%dT%H:%M:%SZ"))
    return last_month




import dataclasses
from typing import ClassVar, List

from datetime import datetime
from functools import reduce
from tqdm import tqdm
import json
import os
import pandas as pd
import wikipedia_tools.scraper.category_extractor as ce
import wikipedia_tools.scraper.revision_extractor as re
from wikipedia_tools.utils import properties
from wikipedia_tools.utils import utils
import random


@dataclasses.dataclass
class WikiPagesRevision:
    output_file: str = "all"
    root_folder: str = properties.ROOT_PATH
    categories: List = dataclasses.field(default_factory=lambda: ["Climate_change"])
    revisions_from: datetime = None
    revisions_to: datetime = datetime.now()
    save_each_page: bool = False
    category_depth: int = 3
    out_version: str = "parquet"  # or jsonl
    page_limit: int = -1
    REVISIONS_FILE_POSTFIX: ClassVar = "revisions.{}"

    # categories_str:str = ""
    def __post_init__(self):
        utils.create_folder(properties.WIKI_SCRAPER_DEBUG_FOLDER)
        self.categories_str = '-'.join(self.categories) if self.categories is not None and len(
            self.categories) > 0 else ""

        if self.out_version not in ["parquet", "jsonl"]: self.out_version = "jsonl"
        WikiPagesRevision.REVISIONS_FILE_POSTFIX = WikiPagesRevision.REVISIONS_FILE_POSTFIX.format(self.out_version)

        self.pages = []
        self.pages = set(
            self.pages + reduce(list.__add__, [ce.get_category_pages(c, self.category_depth) for c in self.categories]))
        print(f'Fetching pages for the following categories: {self.categories}')

        if len(self.pages) == 0:
            raise Exception('No pages to parse. Please specify a proper category or a file with page names.')

        if self.page_limit > 0:
            self.pages = random.sample(self.pages, self.page_limit)
            print(f"page limit is set to {self.page_limit}. Selecting {self.page_limit} random wikipedia pages")

        self.period_str = ""
        if self.revisions_from is not None:
            self.period_str = f'{self.revisions_from.strftime("%d%b%Y")}'
            rev_to = self.revisions_to if self.revisions_to is not None else datetime.now()
            self.period_str = f'{self.period_str}-{rev_to.strftime("%d%b%Y")}'
        print(self._get_filename("test"))

        downloaded_pages = [p for p in self.pages if utils.file_exists(self._get_filename(p))]
        self.pages = [p for p in self.pages if not utils.file_exists(self._get_filename(p))]
        print(f"There are {len(downloaded_pages)} downloaded pages and {len(self.pages)} pages to download")

    def download(self):
        failed_pages = []
        downloaded_pages = []

        print(f'parsing {len(self.pages)} pages')
        if self.save_each_page:
            out_folder = self._get_root_folder("")

            for page in tqdm(self.pages, leave=True):
                if utils.file_exists(self._get_filename(page)):
                    print(f'{page} already downloaded for this time period.')
                    continue

                pages_dict = re.process_page_revisions(page, 'en', self.revisions_from,
                                                       self.revisions_to)

                for page_name, result in pages_dict.items():
                    page_revs, error, disambguities = result

                    if page_revs is None or len(page_revs) == 0:
                        failed_pages.append(page)
                        with open(os.path.join(properties.WIKI_SCRAPER_DEBUG_FOLDER, f"{self.period_str}_failed.txt"),
                                  "a+",
                                  encoding="utf-8") as f:
                            f.write(f"page_name:{page}, error:{error}\n")
                        continue

                    # no error
                    df = pd.DataFrame(page_revs)
                    if "page" not in df.columns.values:
                        print(f"page is not part of the key for page {page}: {page_revs}")
                        continue
                    for page_, group in df.groupby("page"):
                        # save downlaoded page
                        downloaded_pages.append(page_)
                        with open(
                                os.path.join(properties.WIKI_SCRAPER_DEBUG_FOLDER, f"{self.period_str}_downloaded.txt"),
                                "a+",
                                encoding="utf-8") as f:
                            f.write(f"{page_}\n")

                        # get file path for this page
                        file_path = self._get_filename(page_)
                        if self.out_version == "parquet":
                            pd.DataFrame(group).to_parquet(file_path, index=False)
                        else:
                            with open(file_path, 'w', encoding="utf-8") as file:
                                file.write(json.dumps(group.to_json()))


        else:
            out_folder = self._get_filename()
            revisions_df = re.get_revisions_all_pages(self.pages, 'en', self.revisions_from, self.revisions_to)
            with open(out_folder, 'w', encoding="utf-8") as f:
                for r in revisions_df:
                    f.write(json.dumps(r) + "\n")

        print(f"downloaded {len(downloaded_pages)} page revisions out of {(len(failed_pages) + len(downloaded_pages))}")
        return len(downloaded_pages), out_folder

    ###########################
    ######    HELPERS   #######
    ###########################
    def _get_filename(self, page: str = None):

        root = self._get_root_folder(page)

        file_name = os.path.join(
            root,
            f"{self.output_file}_{WikiPagesRevision.REVISIONS_FILE_POSTFIX}"
        )

        utils.create_folder(root)
        return file_name

    def _get_root_folder(self, page: str = None):
        self.output_file = self.output_file if page is None else utils.get_alphanumeric(page)

        root = properties.FOLDER_WIKI_BATCHES.format(self.root_folder, "periodic")
        root = os.path.join(root, self.categories_str) if self.categories_str is not None else root

        if self.revisions_from is not None:
            if page is not None:
                root = os.path.join(root, self.period_str)
            else:
                self.output_file = f"{self.period_str}_{self.output_file}"

        return root



import dataclasses

import pandas as pd

from wikipedia_tools.wikipedia_tools.processor import processor
from wikipedia_tools.utils import properties


@dataclasses.dataclass
class WikipediaRevisionAnalyzer:
    categories : list
    revisions_from : str = "01.01.2001"
    revisions_to : 'str|None' = None
    corpus: str = properties.CORPUS
    download:bool = False

    def __post_init__(self):
        self.wikipages_revisions:pd.DataFrame = self.load_data()



    def get_edited_pages_count(self, period_type:str=""):
        wikipedia_pages_per_category = processor.get_wikipedia_page_titles(categories = self.categories, corpus = self.corpus)
        return

    def get_revisions_count(self):
        return

    def get_revisions_per_user_type_count(self):
        return

    def get_edited_pages_per_user_type_count(self):
        return

    def get_user_type_count(self):
        return



import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib
import ast

from wikipedia_tools.processor import processor
from wikipedia_tools.utils import properties, utils

import textwrap

# Helpers
def barplot(x, y, data, xlabel='Year', ylabel='Count', hue=None, title='', dodge=True):
    sns.set(font_scale=1)
    plt.style.use('fivethirtyeight')
    palette = [sns.color_palette("PuBuGn_d")[0], sns.color_palette("PuBuGn_d")[-3] ]if hue else [
        sns.color_palette("PuBuGn_d")[0]]
    fig, axes = plt.subplots(figsize=(7, 5))
    sns.barplot(x=x, y=y, data=data,
                palette=palette,  # sns.color_palette("PuBuGn_d"),#["#9ecae1", "#cccccc"],
                hue=hue, dodge=dodge
                # palette=('Blues_d' if hue is not None else None)
                )
    plt.title(title, fontsize=14)
    plt.xlabel(xlabel, fontsize=12)
    plt.ylabel(ylabel, fontsize=12)

    for label in axes.xaxis.get_ticklabels():
        label.set_rotation(60)

    axes.yaxis.set_major_formatter(matplotlib.ticker.StrMethodFormatter('{x:,.0f}'))

    plt.show()


def get_edited_page_count(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', plot=True, save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)
    dict_ = [{'period': period, 'article_count': len(set(period_df['title'].unique()))} for period, period_df in
             periodic_df.groupby(['period'])]

    result_df = pd.DataFrame(dict_)
    if plot:
        barplot('period', 'article_count', result_df, ylabel="Count of unique Wikipages", xlabel=desc.capitalize(),
                title="{}ly Count of Edited/Created Climate Change Wikipedia Pages".format(desc.capitalize()))

    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_edited-and-created_wikipages.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


# get_edited_page_count
def get_created_page_count(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', plot=True, save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)

    articles = []
    result = []
    for period_, df_ in periodic_df.groupby('period'):
        period_articles = df_['title'].unique().tolist()
        count_new_articles = len([x for x in period_articles if x not in articles])
        articles.extend(period_articles)
        if len(articles) == 0:
            print('error')
        row = {'period': period_, 'new_article_count': count_new_articles}
        result.append(row)
    result_df = pd.DataFrame(result)
    if plot:
        barplot('period', 'new_article_count', result_df, ylabel="Count of Created Wikipages", xlabel=desc.capitalize(),
                title="{}ly Count of Created Climate Change Wikipedia Pages".format(desc.capitalize()))

    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_created_wikipages.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return pd.DataFrame(result)


def get_revisions_over_time(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', plot=True,
                            save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)

    dict_ = [{'period': period_, 'revision_count': df_['revision_count'].sum()} for period_, df_ in
             periodic_df.groupby(['period'])]
    result_df = pd.DataFrame(dict_)

    if plot:
        barplot('period', 'revision_count', result_df, ylabel="Count of Wikipage Revisions", xlabel=desc.capitalize(),
                title="{}ly Count of Wikipedia Revisions".format(desc.capitalize()))
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_wikipedia_page_revisions.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


def get_words_over_time(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', plot=True, save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)
    periodic_df.sort_values(by='timestamp', inplace=True)

    pre_period_content_dict = {}
    result_arr = []

    for period_, df_ in periodic_df.groupby(['period']):
        current_title_content = {row['title']: str(row['content']) for _, row in df_.iterrows()}
        pre_period_content_dict.update(current_title_content)

        period_word_num = {'period': period_,
                           "word_count": sum([len(v.split()) for _, v in pre_period_content_dict.items()])}
        result_arr.append(period_word_num)

    result_df = pd.DataFrame(result_arr)
    if plot:
        barplot('period', 'word_count', result_df, ylabel="Count of Total Words", xlabel=desc.capitalize(),
                title="{}ly Wikipedia Page Word Count".format(desc.capitalize()))
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_wikipedia_page_words.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")

    return result_df


def get_users_over_time(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', plot=True, save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)
    result_arr = []
    over_time_anonymous = []
    over_time_registered = []
    for period_, df_ in periodic_df.groupby(['period']):
        anonymous = []
        registered = []

        for user_lst_wiki in df_['users'].values.tolist():
            user_lst = ast.literal_eval(user_lst_wiki)

            anonymous.extend([x for x in user_lst if utils.is_ip(x)])
            registered.extend([x for x in user_lst if not utils.is_ip(x)])

        result_arr.append({'period': period_,
                           'count_edits': len(registered),
                           'count_unique_users': len(set(registered)),
                           'User Type': 'Registered User'})
        result_arr.append({'period': period_,
                           'count_edits': len(anonymous),
                           'count_unique_users': len(set(anonymous)),
                           'User Type': 'Anonymous User'})
        over_time_anonymous.extend(anonymous)
        over_time_registered.extend(registered)

    print('Total of Unique Anonymous Users:  ', len(set(over_time_anonymous)))
    print('Total of Unique Registered Users: ', len(set(over_time_registered)))

    result_df = pd.DataFrame(result_arr)
    if plot:
        barplot('period', 'count_edits', result_df, ylabel="Count of Revisions", xlabel=desc.capitalize(),
                title="{}ly edits count of Wikipedia Pages Per user type".format(desc.capitalize()),
                hue="User Type", dodge=True)
        barplot('period', 'count_unique_users', result_df, ylabel="Count of Unique Users", xlabel=desc.capitalize(),
                title="{}ly Count of Wikipedia Page Users".format(desc.capitalize()), hue="User Type", dodge=True)

    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_count_wikipedia_page_users.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


def get_most_edited_articles(top=3, only_popular=False, period=properties.DATE_YEAR, desc='YEAR', plot=True):
    sns.set(font_scale=1)
    plt.style.use('fivethirtyeight')
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)
    num_period = len(periodic_df['period'].unique())
    f, axes = plt.subplots(1, num_period, sharex=False, sharey=True, figsize=(20, 6))

    idx = 0
    for period_, df_ in periodic_df.groupby(['period']):
        top_titles_df = df_[['title', 'revision_count']] \
                            .groupby('title').sum() \
                            .sort_values(by='revision_count', ascending=False) \
                            .reset_index()[0:top]

        ax = axes[idx]
        f.add_subplot(axes[idx])
        sns.barplot(x="title", y="revision_count", data=top_titles_df,
                    color=sns.color_palette("PuBuGn_d", num_period)[idx])

        # labels = [textwrap.fill(label.get_text(), 15) for label in ax.get_xticklabels()]
        # ax.set_xticklabels(labels)
        plt.gca().set_xticklabels(ax.get_xticklabels(), rotation=90, ha="right", fontsize=10)
        plt.gca().set_title(period_, fontsize=14)
        plt.gca().set_xlabel('')
        plt.gca().set_ylabel('')

        idx = idx + 1

    # plt.tight_layout()
    plt.subplots_adjust(wspace=1, hspace=2)
    plt.margins(x=0, y=0)
    # f.text(1, -0.32, 'Year', ha='center')
    f.text(0.5, 1, 'Top {} Titles per {}'.format(top, desc.capitalize()), ha='center', fontsize=16)
    plt.show()


def get_periodic_most_to_least_revised(only_popular=False, period=properties.DATE_YEAR, desc='YEAR',
                                       save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period,
                                                                 desc=desc)
    num_period = len(periodic_df['period'].unique())

    _dict = {}
    for period_, df_ in periodic_df.groupby(['period']):
        sorted_titles = df_[['title', 'revision_count']] \
            .groupby('title').sum() \
            .sort_values(by='revision_count', ascending=False) \
            .reset_index()
        _dict[period_] = sorted_titles['title'].values.tolist()

    max = -1
    for _, v in _dict.items():
        max = len(v) if max < len(v) else max
    extended_dict = {k: v + ([''] * (max - len(v))) for k, v in _dict.items()}

    result_df = pd.DataFrame(extended_dict)
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_most-to-least_revised_wikipedia_pages.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")
    return result_df


def get_attr_for_period_as_txt(period_val,
                               attr_val: str = "links",
                               only_popular=False,
                               period=properties.DATE_YEAR,
                               desc='YEAR') -> []:  ## could be categories or urls

    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)
    periodic_df = periodic_df[periodic_df['period'] == period_val].copy()
    periodic_df.sort_values(by=['revision_count', 'title'], inplace=True, ascending=False)

    result_arr = []
    for _, row in periodic_df.iterrows():
        lst_ = ast.literal_eval(row[attr_val])
        lst_ = [x.replace(" ", "") for x in lst_]
        if len(lst_)>0:
            result_arr.append(". ".join(lst_))

    utils.create_folder(properties.STATS_FOLDER)
    with open(f'{properties.STATS_FOLDER}/{attr_val}_{period_val}.txt', 'w', encoding="utf-8") as f:
        for line in result_arr:
            f.write(f"{line}\n")
    return result_arr


def get_periodic_revisions_percentage(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', save: bool = False):
    periodic_df = processor.get_wikipedia_page_periodic_overview(only_popular=only_popular, period=period, desc=desc)
    periodic_df.sort_values(by=['period', 'revision_count', 'title'], inplace=True, ascending=False)

    periodic_df['revision_percentage'] = round(
        100 * periodic_df['revision_count'] / periodic_df.groupby('period')['revision_count'].transform('sum'), 3)
    result_df = periodic_df[['period', 'title', 'revision_count', 'revision_percentage', 'timestamp']]
    if save:
        utils.create_folder(properties.STATS_FOLDER)
        name = "{}ly_wikipedia_page_revisions_w_percentage.csv".format(desc.lower())
        result_df.to_csv(f"{properties.STATS_FOLDER}/{name}")

    return result_df




from mediawiki import MediaWiki
from functools import reduce

MAX_PAGES = 1000 # this is the max

wikipedia = MediaWiki()

def get_category_pages(category, depth=3):
    pages, subcategories = wikipedia.categorymembers(category, MAX_PAGES)
    if (depth == 0) or (len(subcategories) == 0):
        return pages
    else:
        return pages + reduce(list.__add__, [get_category_pages(c, depth - 1) for c in subcategories])



import os
import traceback
from datetime import datetime

from wikipedia_tools.scraper.wikirevparser_with_time import ProcessRevisions

from wikipedia_tools.base.wikipedia_w_time.exceptions import DisambiguationError
from functools import reduce
from tqdm import tqdm

from wikipedia_tools.utils import properties

start_collecting = False


def process_page_revisions(page_name, lang, rev_from: datetime = None, rev_to: datetime = None):
    res = None
    error = False
    disambiguity = {}
    data = None
    try:

        parser_instance = ProcessRevisions(lang, page_name, rev_from, rev_to)
        try:
            parser_instance.wikipedia_page()
        except DisambiguationError as e:
            print(f'..found disambiguity for page *{page_name}*, fetching  revisions for options: {e.options}')
            disambiguity[page_name] = e.options if page_name not in disambiguity.keys() else disambiguity[
                page_name].extend(e.options)
            dict_ = {}
            for p in e.options:
                print(f"Getting page {p}")
                if p != page_name:
                    dict_.update(process_page_revisions(p, lang, rev_from, rev_to))
            return dict_

        data = parser_instance.parse_revisions()
        if data is not None:
            res = [{**{'page': page_name, 'lang': lang, 'timestamp': item[0]}, **dict(item[1])}
                   for item in reversed(data.items())]
        
    except Exception as other_exc:
        error = True
        print(
            f"\t* An unknown exception occurred for page {page_name}: {other_exc.__repr__()}, {traceback.format_exc()}")
    return {page_name: (res, error, disambiguity)}


def get_revisions_all_pages(pages, lang, rev_from, rev_to):
    print('calling *get_revisions_all_pages* for each page')
    revs = []
    no_revs = []
    downloaded = []
    error_pages = []

    for page in tqdm(pages):
        dict_ = process_page_revisions(page, lang, rev_from, rev_to)
        for page_name, result in dict_.items():
            processed, error, disambiguities = result
            if processed is not None:
                revs.append(processed)
                downloaded.append(page)
            if error:
                error_pages.append(page)
            elif not error and processed is None:
                no_revs.append(page)
    print("End of *get_revisions_all_pages*.")
    _save_debug(downloaded, no_revs, error_pages, rev_from, rev_to)
    return revs


def _save_debug(downloaded, no_revs, error_pages, rev_from, rev_to):
    prefix = f"from{rev_from.strftime('%d-%m-%Y')}" \
             f"_{rev_to.strftime('%d-%m-%Y')}" \
        if rev_from is not None and rev_to is not None \
        else ""

    with open(os.path.join(properties.WIKI_SCRAPER_DEBUG_FOLDER, f"{prefix}_downloaded.txt"), "w") as f:
        f.write(f"{downloaded}\n")
    with open(os.path.join(properties.WIKI_SCRAPER_DEBUG_FOLDER, f"{prefix}_no_revs.txt"), "w") as f:
        f.write(f"{no_revs}\n")
    with open(os.path.join(properties.WIKI_SCRAPER_DEBUG_FOLDER, f"{prefix}_error_pages.txt"), "w") as f:
        f.write(f"{error_pages}\n")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
	Parses the edit histories of Wikipedia language versions.
"""
import nltk
import re
import string

from collections import Counter, defaultdict, OrderedDict
from wikipedia_tools.base.wikipedia_w_time import wikipedia
from wikipedia_tools.base.wikipedia_w_time import exceptions
from datetime import datetime

class ProcessRevisions:

    def __init__(self, language, event, rev_from: datetime=None, rev_to: datetime=None):

        self.language = language
        self.event = event
        self.revisions = {}
        self.content = ""
        self.links = []
        self.rev_from=rev_from
        self.rev_to=rev_to

    def wikipedia_page(self):
        # Get Wikipedia revision history
        wikipedia.set_lang(self.language)

        try:
            event_pagetitle = '_'.join(self.event.split())
            page = wikipedia.WikipediaPage(event_pagetitle, rev_from=self.rev_from, rev_to=self.rev_to)
            self.page = page
            self.revisions = page.revisions
            return page

        except exceptions.PageError:
            print("An exception was thrown in ProcessRevisions in wikirevparser")
            return None

    def replace_link(self, input_string, link, text):
        # Replace link with text
        if type(input_string) != str:
            return None

        output_string = input_string.replace("[[%s]]" % link, text)
        output_string = output_string.replace("[[%s|%s]]" % (link, text), text)

        return output_string

    def get_caption(self, line):
        # Get captions and links from captions, and replace both by string.
        # Todo: fix bug with "<ref" in caption
        if type(line) != str:
            return None

        if "|alt=" in line:

            elements = line.split("|alt=")[-1].split("|")[1:]
            caption = '|'.join(elements)[:-2]
            self.content = self.content.replace(line, caption)

        else:
            caption = re.search(r"thumb\|(.*)\]\]", line, re.IGNORECASE)
            if not caption: return None
            caption = caption.group(1)
            if "px" in caption or caption.startswith("{{legend"): return None

        # Links
        links, texts = self.get_links(caption)

        for link, text in zip(links, texts):
            caption = self.replace_link(caption, link, text)

            self.content = self.replace_link(self.content, link, text)
            self.links.append(link.lower())

        caption = caption.split("|")[-1].strip()
        caption = self.proper_formatting(caption, punct=True)

        return caption

    def get_category(self, link):
        # Get categories and remove label and language links.
        if type(link) != str:
            return None

        elements = link.split(":")

        if len(elements) == 2:
            label, category = elements

        elif len(elements) == 3:
            label, wiki, category = elements

        else:
            return None
        # Filter out language links
        if len(label) == 2 and label.islower(): return None

        return category

    def get_image_link(self, line):
        # Get image title and return correct link to commons.wikimedia.org
        if type(line) != str:
            return None

        occurrence = re.search(r"(:|=)([^(:|=)].*)(.jpg|.svg|.png)", line, re.IGNORECASE)
        if not occurrence:
            occurrence = re.search(r"(^)([^(:|=)].*)(.jpg|.svg|.png)", line, re.IGNORECASE)
        try:
            image_title, image_extension = occurrence.group(2, 3)
        except AttributeError:
            return None
        if ":" in image_title:
            image_title = image_title.split(":")[-1]

        image_title = '_'.join([x for x in image_title.split()])
        image_link = "https://commons.wikimedia.org/wiki/File:" + image_title + image_extension
        return image_link

    def get_links(self, input_string):
        # Get links from content.

        if type(input_string) != str:
            return None, None

        double_square_bracket, content = self.get_occurrences(r"\[\[(.*?)\]\]", input_string)
        links = []
        texts = []

        for element in double_square_bracket:
            elements = element.split("|")

            link = elements[0]
            if len(elements) == 2:
                text = elements[1]
            else:
                text = link

            links.append(link)
            texts.append(text)

        return links, texts

    def get_occurrences(self, regex, content):
        # Get references/citations.

        if type(content) != str:
            return None

        exp = re.compile(regex, re.IGNORECASE)
        occurrences = exp.findall(content)
        content = re.sub(exp, '', content)

        return occurrences, content

    def get_reference_types(self, input_list):
        # Get the reference type from a reference/citation block if annotated.
        if type(input_list) != list or len(input_list) < 1:
            return None

        elif type(input_list[0]) != str:
            return None

        types = Counter()

        for element in input_list:
            try:
                reference_type = re.search("{{[c|C]ite (.*?)( |\|)", element, re.IGNORECASE).group(1)
                if len(reference_type) <= 1: continue
                types[reference_type] += 1

            except AttributeError:
                types["_nill_"] += 1

        return types

    def get_urls(self, input_list):
        # Get URLs.
        urls = []
        http_regex = "http[s]*:\/\/([^ ]*)"

        for element in input_list:
            url = re.search(http_regex + r"( |}})", element, re.IGNORECASE)
            if url:
                url = url.group(1).split("|")[0]
            else:
                url = re.search(http_regex, element, re.IGNORECASE)
                if not url: continue
                url = url.group(1).split("|")[0]
            if "www" in url:
                url = url[4:]
            urls.append(url)
        return urls

    def parse_images(self):
        # Get and parse images.
        images = []
        captions = []
        extensions = [".jpg", ".svg", ".png", ".JPG", ".SVG", ".PNG"]

        for line in self.content.split("\n"):
            for extension in extensions:

                if not extension in line: continue

                image_link = self.get_image_link(line)
                images.append(image_link)

                self.content = self.content.replace(line, "")

                caption = self.get_caption(line)
                captions.append(caption)

        return images, captions

    def parse_categories(self):
        # Get and parse categories (and links).
        categories = []
        possible_links, texts = self.get_links(self.content)

        for link, text in zip(possible_links, texts):

            if ":" in link:
                category = self.get_category(link)
                if link.startswith("Category:"):
                    categories.append(link.split(":")[1])
                self.content = self.replace_link(self.content, link, "")

            else:
                self.links.append(link.lower())
                self.content = self.replace_link(self.content, link, text)

        return categories

    def parse_references(self):
        # Get and parse references and the top level domains.
        reference_template_types = Counter()  # some references have "type" markup, e.g. "book", "thesis", "news" etc.

        regex_letters = "\dA-Za-zğäåæáéëíıïóøöúü"
        regex_symbols = "\"'(){}[\]&=«»:.,?_~–\-/| "

        references, self.content = self.get_occurrences(r"<ref(.*?)<\/ref>", self.content)
        citations, self.content = self.get_occurrences(r"{{[c|C]ite [" + regex_letters + regex_symbols + ">]+}}",
                                                       self.content)

        urls = self.get_urls(references + citations + [x for x in self.content.split()])

        reference_types = self.get_reference_types(references + citations)
        if reference_types != None:
            reference_template_types += reference_types

        return urls, reference_template_types

    def proper_formatting(self, input_string, punct=True):
        # Add end punct, strip quotation marks, and tokenize.

        if type(input_string) != str:
            return None
        if len(input_string) < 1:
            return None
        if punct:
            if input_string[-1] not in string.punctuation:
                input_string += "."

        input_string = "".join([x for x in input_string if x != "'"])

        try:
            nltk.data.find('tokenizers/punkt')
        except LookupError:
            nltk.download('punkt')

        output_string = ' '.join(nltk.word_tokenize(input_string))

        return output_string

    def parse_sections(self):
        # Get and parse section titles.
        # Todo: improve section organization in output
        sections = []
        header1s, self.content = self.get_occurrences(r"[^=]={2}([^=].*?)={2}", self.content)
        header2s, self.content = self.get_occurrences(r"[^=]={3}([^=].*?)={3}", self.content)
        header3s, self.content = self.get_occurrences(r"[^=]={4}([^=].*?)={4}", self.content)
        sections.append([self.proper_formatting(x.strip(), punct=False) for x in header1s])
        sections.append([self.proper_formatting(x.strip("=").strip(), punct=False) for x in header2s])
        sections.append([self.proper_formatting(x.strip("==").strip(), punct=False) for x in header3s])

        return sections

    def parse_text(self):
        # Get and remove tables and markup from content.
        clean_content = []
        italics_markup, self.content = self.get_occurrences(r"\'{2,}", self.content)

        for line in self.content.split("\n"):

            line = line.strip()
            if len(line) == 0: continue
            if line[0] in string.punctuation or "px" in line: continue

            clean_content.append(self.proper_formatting(line))

        self.content = ' '.join([w for w in clean_content])

    def parse_revisions(self):
        # Input: revisions from Wikipedia page.
        # Output: dictionary with extracted page elements per revision.
        data = OrderedDict([])

        if self.revisions == None: return None

        for n, revision in enumerate(self.revisions):
            self.links = []
            self.content = ""
            parsed_data = defaultdict()
            timestamp = revision["timestamp"]

            if "user" not in revision.keys(): continue
            user = revision["user"]

            if "*" not in revision["slots"]["main"].keys(): continue
            self.content = revision["slots"]["main"]["*"]

            urls, reference_template_types = self.parse_references()
            images, captions = self.parse_images()
            categories = self.parse_categories()
            sections = self.parse_sections()
            self.parse_text()

            # parsed_data["captions"] = captions
            parsed_data["categories"] = categories
            parsed_data["content"] = self.content
            parsed_data["images"] = images
            parsed_data["links"] = self.links
            # parsed_data["reference_template_types"] = reference_template_types
            parsed_data["sections"] = sections
            parsed_data["urls"] = urls
            parsed_data["user"] = user

            data[timestamp] = parsed_data
        return data


import logging

from wikipedia_tools.utils import properties
from wikipedia_tools.utils import utils
from pathlib import Path
from datetime import datetime

import pandas as pd
from glob import glob
import os


def get_wikipedia_page_titles(categories: list = ["Climate_change"], corpus: str = properties.CORPUS):
    wikipedia_pages_df = {}

    for category in categories:
        try:
            all_unique_pages = [Path(x).name.replace(".parquet", "") for x in
                                glob(os.path.join(corpus, category, "*", "*.parquet"))]
            all_unique_pages = list(set(all_unique_pages))

            wikipedia_pages_df[category] = all_unique_pages
            logging.info(f"There are {len(all_unique_pages)} wikipedia pages for the category {category}")
        except:
            logging.error(f"Make sure that the corpus parquet files are located here {os.path.join(corpus, category)}")
    return wikipedia_pages_df


def get_wikipedia_page_data(wikipedia_page: str, category: str = "Climate_change",
                            corpus: str = properties.CORPUS) -> pd.DataFrame:
    def to_tuple(arr):
        return tuple(map(tuple, arr))

    paths = glob(os.path.join(corpus, category, "*", f"{wikipedia_page}.parquet"))
    data_df: pd.DataFrame = None
    _all_data_lst = []
    for path in paths:
        _df = pd.read_parquet(path)

        _df['categories'] = _df['categories'].apply(tuple)
        _df['images'] = _df['images'].apply(tuple)
        _df['links'] = _df['links'].apply(tuple)
        _df['urls'] = _df['urls'].apply(tuple)
        _df['sections'] = _df['sections'].apply(to_tuple)

        _all_data_lst.append(_df)

    if len(_all_data_lst) == 1:
        data_df = _all_data_lst[0]
    elif len(_all_data_lst) > 1:
        data_df = pd.concat(_all_data_lst)
        data_df.drop_duplicates(keep="first", inplace=True)

    return data_df



import pandas as pd

import wikipedia_tools.processor.loader as loader
import wikipedia_tools.utils.properties as prop
import wikipedia_tools.utils.utils as utils
import nlpaf.lexicons as lexicons


## HELPERS

def _add_id(row):
    row['numeric_id'] = int(row['Filename'].split('_')[1].split('.')[0])
    return row

## END OF HELPERS


def extract_nrc_emotion_for_all_revisions(wiki_page_title, content_col='content', save = False):

    file_path = prop.FEATURE_NRC_PATH.format(utils.get_alphanumeric(wiki_page_title))

    if utils.file_exists(file_path):
        return pd.read_csv(file_path, index_col='numeric_id')

    print('file not saved, getting original data...')
    contents_df = loader.get_wiki_page(wiki_page_title)#ADD GET CONTENT HERE

    nrc_df = contents_df[[content_col]].copy()

    print('calculating lexicon cound for nrc...')
    nrc_df = lexicons.count_nrc_emotions_and_sentiments(nrc_df, text_column=content_col)

    if save:
        nrc_df.reset_index().to_csv(file_path, index=False)
    return nrc_df

def extract_nrc_emotion(df, content_cols, file_path = None):

    data_df = df[content_cols].copy()
    nrc_res_arr = []
    for col in content_cols:
        print('calculating lexicon count for nrc...')
        nrc_col_df = lexicons.count_nrc_emotions_and_sentiments(data_df, text_column=col, prefix='nrc_'+col+'_')
        nrc_res_arr.append(nrc_col_df)
    nrc_df = pd.concat(nrc_res_arr, axis=1, join="inner")
    if file_path is not None:
        nrc_df.reset_index().to_csv(file_path, index=False)
    return nrc_df

def extract_empath(wiki_page_title, content_col='content', file_path = None):

    file_path = prop.FEATURE_EMPATH_PATH.format(utils.get_alphanumeric(wiki_page_title))

    if utils.file_exists(file_path):
        return pd.read_csv(file_path, index_col='numeric_id')

    print('file not saved, getting original data...')
    contents_df = loader.get_wiki_page(wiki_page_title)#ADD GET CONTENT HERE

    empath_df = contents_df[[content_col]].copy()


    print('calculating lexicon count for empath...')
    empath_df = lexicons.count_empath(empath_df, text_column=content_col,prefix='empath_')
    empath_df.reset_index().to_csv(file_path, index=False)
    return empath_df



def extract_mpqa_arg(df, content_cols, file_path = None):
    file_path = prop.FEATURE_MPQA_ARG_PATH.format(file_path)

    if utils.file_exists(file_path):
        return pd.read_csv(file_path)

    data_df = df[content_cols].copy()
    mpqa_arg_arr = []
    for col in content_cols:
        print('calculating lexicon count for mpqa arg...')
        nrc_col_df = lexicons.count_mpqa_arg(data_df, text_column=col, prefix='mpqa_arg_'+col+'_')
        mpqa_arg_arr.append(nrc_col_df)
    mpqa_arg_df = pd.concat(mpqa_arg_arr, axis=1, join="inner")
    if file_path is not None:
        mpqa_arg_df.reset_index().to_csv(file_path, index=False)


    return mpqa_arg_df


# This copy is forked and copied from here  https://github.com/ajoer/Wikipedia
# which is forked from here https://github.com/goldsmith/Wikipedia
from .wikipedia import *
from .exceptions import *

__version__ = (1, 4, 0)



# Files and folders paths)
import os
ROOT_PATH = os.path.join(os.path.dirname(__file__), '..', '..', '..')
CORPUS = os.path.join("V:", os.sep, "InsightsNet", "Corpora", "Wikipedia", "wikipedia_dump")
WIKI_PAGES =os.path.join(ROOT_PATH,  "data", "wiki_pages")
OUT= os.path.join( ROOT_PATH , "data", "generated")



STATS_FOLDER = os.path.join(ROOT_PATH,  "data",  "stats")

WIKI_TITLES_PATH = os.path.join(  ROOT_PATH , "data", "docs", "wiki_pages_titles.txt")
WIKI_CLIMATE_CHANGE_POPULAR_PAGES = os.path.join( ROOT_PATH , "data", "docs", "wikipedia_wikiproject_climate_change_popular_pages.csv")
WIKI_MULTI_PAGES = os.path.join(  ROOT_PATH , "data", "docs", "multi_pages.txt")

WIKI_OVERVIEW_CSV = os.path.join( OUT ,  "wiki_pages_overview.csv")
WIKI_EDITS_COUNT_CSV= os.path.join( OUT ,  "wiki_pages_edit_number_{}_{}.csv")
WIKI_EDITS_PERIODIC_CSV= os.path.join( OUT ,  "wiki_pages_edit_{}_{}.csv")
FOLDER_WIKI_BATCHES= os.path.join( "{}", "data" ,  "{}_wiki_batches")

WIKI_SCRAPER_DEBUG_FOLDER= os.path.join( ROOT_PATH, "data" ,  "debug")

# Extracted features)
EXTRACTED_FEATURES_PATH = os.path.join( OUT  ,  "extracted_features", "")
FIG_PATH = os.path.join( ROOT_PATH , "data", "figures", "")

FEATURE_LIWC_PATH = os.path.join( OUT  ,  "extracted_features", "liwc_{}.csv")
RAW_LIWC_PATH = os.path.join( OUT  ,  "extracted_features", "debateorg_arguments_txt", "liwc_{}.csv")

FEATURE_NRC_PATH = os.path.join( OUT  ,  "extracted_features", "nrc_{}.csv")
FEATURE_MPQA_ARG_PATH = os.path.join( OUT  ,  "extracted_features", "mpqa_arg_{}.csv")
FEATURE_EMPATH_PATH = os.path.join( OUT  ,  "extracted_features", "empath_{}.csv")
FEATURE_POLITENESS_PATH = os.path.join( OUT  ,  "extracted_features", "politeness_{}.csv")

# CONSTANTS
DATE_MONTHLY = "%m-%Y"
DATE_YEAR = "%Y"


import datetime
import logging

import pandas as pd
import os

from glob import glob
from tqdm import tqdm

from wikipedia_tools.utils import properties, utils
from wikipedia_tools.processor import loader
from pathlib import Path


def get_wiki_popular_pages():
    wiki_popular_page_df = pd.read_csv(properties.WIKI_CLIMATE_CHANGE_POPULAR_PAGES)
    return wiki_popular_page_df[wiki_popular_page_df['Page title'].notna()]


def is_popular(title):
    popular_df = get_wiki_popular_pages()
    return len(popular_df[popular_df['Page title'] == title]) > 0


def is_important(title):
    popular_df = get_wiki_popular_pages()
    return popular_df[popular_df['Page title'] == title]['Importance'].values[
        0] if is_popular(title) else 'na'


def get_wiki_pages_overview(category="Climate_change"):
    """
    Processes all wikipages and generates 1 dataframe where each row represents 1 wikipage with info:
    page title, created on, last updated, initial content, latest content, edits numner, content edits count, unique users,
    is_popular (if it exists under Wikipedida Climate Change Project), is_important (if it is popular and flagged important by the wiki project)
    :return:
    """
    path = Path(properties.WIKI_OVERVIEW_CSV)
    if path.is_file():
        return pd.read_csv(properties.WIKI_OVERVIEW_CSV, )

    wiki_titles = loader.get_wikipedia_page_titles(categories=[category], corpus=properties.CORPUS)
    popular_pages_df = get_wiki_popular_pages()
    wiki_page_meta_arr = []

    print('there are {} wiki titles'.format(len(wiki_titles)))
    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)

        if original_df is None:
            continue
        if len(original_df) == 0:
            print('{} has len 0'.format(title_code))
            continue
        _df = original_df.copy()

        wiki_page_meta = {}

        _df['timestamp_str'] = _df['timestamp']
        _df['timestamp'] = pd.to_datetime(_df['timestamp'], infer_datetime_format=True)
        _df = _df.sort_values(by="timestamp")

        init_content_df = _df[(~_df['content'].isna()) & (_df['content'] != '')].copy()
        init_content_df = init_content_df[init_content_df['timestamp'] == init_content_df['timestamp'].min()]
        if len(init_content_df) == 0:
            print('NO CONTENT: ', title_code)
            continue
        last_content = _df[_df['timestamp'] == _df['timestamp'].max()]['content'].values[0]
        edits_w_latest_content_df = (_df[(_df['content'] == last_content)])
        last_updated_df = edits_w_latest_content_df[
            edits_w_latest_content_df['timestamp'] == edits_w_latest_content_df['timestamp'].min()]

        wiki_page_meta['title_code'] = title_code
        wiki_page_meta['page_title'] = init_content_df['page'].values[0]
        wiki_page_meta['created_on'] = init_content_df['timestamp'].values[0]
        wiki_page_meta['init_content'] = init_content_df['content'].values[0]
        wiki_page_meta['last_updated'] = last_updated_df['timestamp'].values[0]
        wiki_page_meta['latest_content'] = last_updated_df['content'].values[0]
        wiki_page_meta['edits_number'] = (len(original_df['timestamp'].unique()))
        wiki_page_meta['content_edits_number'] = (
            len(_df.drop_duplicates(subset=['content'], keep='first', ignore_index=True)))
        wiki_page_meta['unique_users'] = len(original_df['user'].unique())

        wiki_page_meta['is_popular'] = is_popular(init_content_df['page'].values[0])
        wiki_page_meta['importance'] = popular_pages_df[popular_pages_df['Page title'] == title_code]['Importance'].values[
            0] if wiki_page_meta['is_popular'] else 'na'

        wiki_page_meta_arr.append(wiki_page_meta)

    meta_df = pd.DataFrame(wiki_page_meta_arr)
    meta_df.to_csv(properties.WIKI_OVERVIEW_CSV, index=False)
    return meta_df

#get_wiki_edits_num
def get_revision_count(only_popular=False, date_granularity=properties.DATE_MONTHLY, desc='MONTH-YEAR', category="Climate_change"):
    fname = properties.WIKI_EDITS_COUNT_CSV.format(('POPULAR' if only_popular else 'ALL'), desc)
    path = Path(fname)
    if path.is_file():
        return pd.read_csv(fname, index_col='abstracted_date')

    wiki_titles = loader.get_wikipedia_page_titles(categories=[category], corpus=properties.CORPUS)

    print('there are {} wiki titles'.format(len(wiki_titles)))

    result_df = pd.DataFrame()

    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)
        title = original_df["page"].values[0]
        if (only_popular and not is_popular(title)) or (original_df is None) or len(original_df) == 0:
            continue

        _df = original_df.copy()

        _df['timestamp_str'] = _df['timestamp']
        _df['timestamp'] = pd.to_datetime(_df['timestamp'], infer_datetime_format=True)
        _df['abstracted_date'] = _df['timestamp'].dt.strftime(date_granularity)

        edits_count_df = _df['abstracted_date'].value_counts().to_frame(name='edits_count')
        edits_count_df.index.name = 'abstracted_date'
        result_df = pd.concat([result_df, edits_count_df]).groupby(['abstracted_date']).sum()

    result_df.reset_index().to_csv(fname, index=False)
    return result_df


#

def batch_revisions_per_period(period=properties.DATE_MONTHLY, desc='monthly', category="Climate_change", corpus=None):
    folder_root = properties.FOLDER_WIKI_BATCHES.format(desc)
    corpus=properties.CORPUS if  corpus  is None else corpus
    wiki_titles = loader.get_wikipedia_page_titles(categories=[category], corpus=corpus)

    print('there are {} wiki titles'.format(len(wiki_titles)))
    periords_str_lst = []
    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)
        if (original_df is None) or len(original_df) == 0:
            continue
        _df = original_df.copy()

        _df['timestamp_str'] = _df['timestamp']
        _df['timestamp'] = pd.to_datetime(_df['timestamp'], infer_datetime_format=True)
        _df['abstracted_date'] = _df['timestamp'].dt.strftime(period)
        periords_str_lst.extend(_df['abstracted_date'].unique().tolist())
        for period_val, period_df in _df.groupby('abstracted_date'):
            destination = f"{folder_root}/{period_val}/{title_code}.parquet"
            utils.create_folder(f"{folder_root}/{period_val}")
            period_df.to_parquet(destination, index=False)
    periods_str_lst = list(set(periords_str_lst))
    periods_str_lst = sorted([datetime.strptime(x, period) for x in  periods_str_lst])

    return folder_root,periods_str_lst

# get_wiki_edits_per_period
def get_wikipedia_page_periodic_overview(only_popular=False, period=properties.DATE_YEAR, desc='YEAR', category="Climate_change"):
    fname = properties.WIKI_EDITS_PERIODIC_CSV.format(('POPULAR' if only_popular else 'ALL'), desc)
    path = Path(fname)
    if path.is_file():
        return pd.read_csv(fname)

    wiki_titles = loader.get_wikipedia_page_titles(categories=[category], corpus=properties.CORPUS)

    print('there are {} wiki titles'.format(len(wiki_titles)))

    result_arr = []
    for title_code in tqdm(wiki_titles[category]):
        original_df = loader.get_wikipedia_page_data(title_code)
        if (only_popular and not is_popular(original_df["page"].values[0])) or (original_df is None) or len(original_df) == 0:
            continue
        _df = original_df.copy()

        _df['timestamp_str'] = _df['timestamp']
        _df['timestamp'] = pd.to_datetime(_df['timestamp'], infer_datetime_format=True)
        _df['abstracted_date'] = _df['timestamp'].dt.strftime(period)

        for period_val, period_df in _df.groupby('abstracted_date'):
            latest_df = period_df[period_df['timestamp'] == period_df['timestamp'].max()]
            revision_per_period = {}
            revision_per_period['content'] = latest_df['content'].values[0]
            revision_per_period['title'] = latest_df['page'].values[0]
            revision_per_period['title_code'] = title_code
            revision_per_period['period'] = period_val
            revision_per_period['user_count'] = len(period_df['user'].unique())
            revision_per_period['users'] = period_df['user'].values.tolist()
            revision_per_period['revision_count'] = len(period_df)
            revision_per_period['timestamp'] = latest_df['timestamp'].values[0]
            revision_per_period['links'] = latest_df['links'].values[0]
            revision_per_period['categories'] = latest_df['categories'].values[0]
            revision_per_period['urls'] = latest_df['urls'].values[0]
            result_arr.append(revision_per_period)

    result_df = pd.DataFrame(result_arr)  # , index = ['period', 'title'])

    result_df.to_csv(fname, index=False)
    return result_df

